library IEEE;                          
use IEEE.std_logic_1164.all;                          
use work.conv_pkg.all;                          
                          
entity warplab_siso_opbw is                          
  generic (                          
    C_BASEADDR: std_logic_vector(0 to 31) := X"00000000";                          
    C_HIGHADDR: std_logic_vector(0 to 31) := X"0003FFFF";                          
    C_OPB_AWIDTH: integer := 32;                          
    C_OPB_DWIDTH: integer := 32                          
  );                          
  port (                          
    ce: in std_logic;                          
    opb_abus: in std_logic_vector(0 to 31);                          
    opb_be: in std_logic_vector(0 to 3);                          
    opb_clk: in std_logic;                          
    opb_dbus: in std_logic_vector(0 to 31);                          
    opb_rnw: in std_logic;                          
    opb_rst: in std_logic;                          
    opb_select: in std_logic;                          
    opb_seqaddr: in std_logic;                          
    radio2_adc_i: in std_logic_vector(0 to 13);                          
    radio2_adc_i_otr: in std_logic;                          
    radio2_adc_q: in std_logic_vector(0 to 13);                          
    radio2_adc_q_otr: in std_logic;                          
    radio2_rssi: in std_logic_vector(0 to 9);                          
    startcapture: in std_logic;                          
    debug_capturing: out std_logic;                          
    debug_transmitting: out std_logic;                          
    radio2_dac_i: out std_logic_vector(0 to 15);                          
    radio2_dac_q: out std_logic_vector(0 to 15);                          
    rssi_adc_clk: out std_logic;                          
    sgp_dbus: out std_logic_vector(0 to 31);                          
    sgp_errack: out std_logic;                          
    sgp_retry: out std_logic;                          
    sgp_toutsup: out std_logic;                          
    sgp_xferack: out std_logic                          
  );                          
end warplab_siso_opbw;                          
                          
architecture structural of warplab_siso_opbw is                          
  signal ce_x0: std_logic;                          
  signal clk: std_logic;                          
  signal debug_capturing_x0: std_logic;                          
  signal debug_transmitting_x0: std_logic;                          
  signal opb_abus_x0: std_logic_vector(31 downto 0);                          
  signal opb_be_x0: std_logic_vector(3 downto 0);                          
  signal opb_dbus_x0: std_logic_vector(31 downto 0);                          
  signal opb_rnw_x0: std_logic;                          
  signal opb_rst_x0: std_logic;                          
  signal opb_select_x0: std_logic;                          
  signal opb_seqaddr_x0: std_logic;                          
  signal radio2_adc_i_otr_x0: std_logic;                          
  signal radio2_adc_i_x0: std_logic_vector(13 downto 0);                          
  signal radio2_adc_q_otr_x0: std_logic;                          
  signal radio2_adc_q_x0: std_logic_vector(13 downto 0);                          
  signal radio2_dac_i_x0: std_logic_vector(15 downto 0);                          
  signal radio2_dac_q_x0: std_logic_vector(15 downto 0);                          
  signal radio2_rssi_x0: std_logic_vector(9 downto 0);                          
  signal rssi_adc_clk_x0: std_logic;                          
  signal sgp_dbus_x0: std_logic_vector(31 downto 0);                          
  signal sgp_errack_x0: std_logic;                          
  signal sgp_retry_x0: std_logic;                          
  signal sgp_toutsup_x0: std_logic;                          
  signal sgp_xferack_x0: std_logic;                          
  signal startcapture_x0: std_logic;                          
                          
begin                          
  ce_x0 <= ce;                          
  opb_abus_x0 <= opb_abus xor C_BASEADDR;
  opb_be_x0 <= opb_be;                          
  clk <= opb_clk;                          
  opb_dbus_x0 <= opb_dbus;                          
  opb_rnw_x0 <= opb_rnw;                          
  opb_rst_x0 <= opb_rst;                          
  opb_select_x0 <= opb_select;                          
  opb_seqaddr_x0 <= opb_seqaddr;                          
  radio2_adc_i_x0 <= radio2_adc_i;                          
  radio2_adc_i_otr_x0 <= radio2_adc_i_otr;                          
  radio2_adc_q_x0 <= radio2_adc_q;                          
  radio2_adc_q_otr_x0 <= radio2_adc_q_otr;                          
  radio2_rssi_x0 <= radio2_rssi;                          
  startcapture_x0 <= startcapture;                          
  debug_capturing <= debug_capturing_x0;                          
  debug_transmitting <= debug_transmitting_x0;                          
  radio2_dac_i <= radio2_dac_i_x0;                          
  radio2_dac_q <= radio2_dac_q_x0;                          
  rssi_adc_clk <= rssi_adc_clk_x0;                          
  sgp_dbus <= sgp_dbus_x0;                          
  sgp_errack <= sgp_errack_x0;                          
  sgp_retry <= sgp_retry_x0;                          
  sgp_toutsup <= sgp_toutsup_x0;                          
  sgp_xferack <= sgp_xferack_x0;                          
                          
  sysgen_dut: entity work.warplab_siso_cw                          
    port map (                          
      ce => ce_x0,                          
      clk => clk,                          
      opb_abus => opb_abus_x0,                          
      opb_be => opb_be_x0,                          
      opb_dbus => opb_dbus_x0,                          
      opb_rnw => opb_rnw_x0,                          
      opb_rst => opb_rst_x0,                          
      opb_select => opb_select_x0,                          
      opb_seqaddr => opb_seqaddr_x0,                          
      radio2_adc_i => radio2_adc_i_x0,                          
      radio2_adc_i_otr => radio2_adc_i_otr_x0,                          
      radio2_adc_q => radio2_adc_q_x0,                          
      radio2_adc_q_otr => radio2_adc_q_otr_x0,                          
      radio2_rssi => radio2_rssi_x0,                          
      startcapture => startcapture_x0,                          
      debug_capturing => debug_capturing_x0,                          
      debug_transmitting => debug_transmitting_x0,                          
      radio2_dac_i => radio2_dac_i_x0,                          
      radio2_dac_q => radio2_dac_q_x0,                          
      rssi_adc_clk => rssi_adc_clk_x0,                          
      sgp_dbus => sgp_dbus_x0,                          
      sgp_errack => sgp_errack_x0,                          
      sgp_retry => sgp_retry_x0,                          
      sgp_toutsup => sgp_toutsup_x0,                          
      sgp_xferack => sgp_xferack_x0                          
    );                          
                          
end structural;                          
